import 'dart:async';

import 'package:equatable/equatable.dart';
import 'package:logging/logging.dart';
import 'package:meta_forms/meta_forms.dart';
import 'package:sunny_sdk_core/sunny_sdk_core.dart';
import 'package:sunny_service_stubs/models/facts/facts.dart';

class MetaOrgService {
  static final log = Logger("metaOrgService");
  final MSchemaService schemaService;
  final Map<MSchemaRef?, OrganizationMetadata> _orgTypes = {};

  MetaOrgService({required this.schemaService}) {
    schemaService.loadedEntitySchemas.forEach((schema) async {
      if (schema.schemaRef == OrganizationRef ||
          schema.parentRef == OrganizationRef) {
        try {
          log.fine("Found org ${schema.schemaRef}");
          final org = await _loadOrgSchema(schema.parentRef, schema)!;
          _orgTypes[schema.schemaRef] = org;
        } catch (e, stack) {
          log.info("Error loading org schema: $e", e, stack);
        }
      }
    });
    schemaService.load();
  }

  Future<OrganizationMetadata> getOrgMetadata(MSchemaRef ref,
      {bool reload = false}) async {
    var existing = _orgTypes[ref];
    if (existing == null) {
      final schema = await schemaService.getSchema(ref, reload: reload);
      existing = await _loadOrgSchema(ref, schema as IMEntityDefinition);
      _orgTypes[ref] = existing!;
    }
    return existing;
  }

  Future<OrganizationMetadata>? _loadOrgSchema(
      MSchemaRef? ref, IMEntityDefinition entity) {
    return null;
  }
}

class OrganizationMetadata extends Equatable {
  final IMEntityDefinition organization;
  final Map<String, MembershipMetadata> memberships;

  OrganizationMetadata(this.organization, this.memberships);

  @override
  List<Object> get props => [organization, memberships];

  MSchemaRef? get schemaRef => organization.schemaRef;
}

enum MembershipCardinality { singleton, multi }

class MembershipMetadata extends Equatable {
  final IMEntityDefinition membership;
  final MembershipCardinality type;
  final String name;
  final MSchemaRef orgType;

  MembershipMetadata(this.membership, this.type, this.orgType, this.name);

  @override
  List<Object> get props => [membership, type, name, orgType];
}
