import 'package:flutter/widgets.dart';
import 'package:provider/provider.dart';
import 'package:sunny_essentials/sunny_essentials.dart';

class SharedValue<T> extends StatelessWidget {
  final T? initialValue;
  final Widget child;

  const SharedValue({Key? key, this.initialValue, required this.child})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return ChangeNotifierProvider<ValueNotifier<T?>>(
      create: (_) {
        return ValueNotifier<T?>(initialValue);
      },
      child: child,
    );
  }
}

typedef SharedWidgetBuilder<T> = Widget Function(BuildContext? context, T value);

class SharedBuilder<T> extends StatefulWidget {
  final SharedWidgetBuilder<T>? builder;

  const SharedBuilder({Key? key, this.builder}) : super(key: key);

  @override
  _SharedBuilderState<T> createState() => _SharedBuilderState<T>();
}

class _SharedBuilderState<T> extends State<SharedBuilder<T?>> {
  bool? _isInitialized;
  Widget? built;
  ValueNotifier<T?>? _notifier;
  BuildContext? _context;

  @override
  Widget build(BuildContext context) {
    _context = context;
    _init();
    return built ??= widget.builder!(_context, _notifier?.value);
  }

  _init() {
    if (_isInitialized == true) return;

    _notifier ??= _context!.find<ValueNotifier<T?>>();
    if (_notifier != null) {
      _isInitialized = true;
      _notifier!.addListener(_onValueChange);
    }
  }

  _onValueChange() {
    setState(() => built = null);
  }

  @override
  void initState() {
    super.initState();
    _context = context;
    _init();
  }

  @override
  void dispose() {
    _notifier?.removeListener(_onValueChange);
    super.dispose();
  }
}
